// File: Classifier.h
// Purpose: Definition of the Classifier class
// Remarks: This class is an abstract base class.
// Author: Yihua Chen (yhchen@u.washington.edu)
// Last modified: 9/13/2007 12:11:09 PM

#ifndef IDL_CLASSIFIER_H
#define IDL_CLASSIFIER_H

#include <vector>
#include "SimData.h"

namespace idl
{
	/** Abstract base class for classifiers. */
	class Classifier
	{
	public:
		virtual ~Classifier() {} /**< Destructor. */

		/**
		 * Classify the test samples of the similarity data set.
		 * @param[in]	data		similarity data set
		 * @param[out]	results		vector to store the estimated class labels
		 * @return		true if success; false if failure
		 * @attention	This is a pure virtual function.
		 */
		virtual bool classify(const SimData &data, std::vector<int> &results) = 0;
	};
}

#endif // IDL_CLASSIFIER_H
