// File: NearestMean.cpp
// Purpose: Implementation of the NearestMean class
// Remarks: This class is designed for nearest-mean classification.
// Author: Yihua Chen (yhchen@u.washington.edu)
// Last modified: 1/5/2008 11:43:37 PM

//#include <cstdlib>
//#include <ctime>
#include <iostream>
#include "CmRandomNumberGenerator.h" // Mersenne Twister (singleton) header file
#include "NearestMean.h"

using namespace std;

namespace idl
{
	// Classify the test samples of the similarity data set
	bool NearestMean::classify(const SimData &data, vector<int> &results)
	{
		vector<double> meanSim;
		vector<int> classLabels;

		// Get the number of test samples
		unsigned int numTestSamples = data.getTestSetSize();
		results.resize(numTestSamples);

		for (unsigned int i = 0; i < numTestSamples; ++i)
		{
			if (parameters.globalFlag)
			{
				data.getMeanSim(i, meanSim, classLabels);
			}
			else
			{
				data.getMeanSim(i, meanSim, classLabels, parameters.neighborhoodSize);
			}

			results[i] = vote(meanSim, classLabels);
		}
		
		return true;
	}

	// Vote according to the mean similarity of each class
	int NearestMean::vote(const vector<double> &meanSim, const vector<int> &classLabels)
	{
		double maxMeanSim = 0.0;

		for (vector<double>::const_iterator itr = meanSim.begin(); itr != meanSim.end(); ++itr)
		{
			if (*itr > maxMeanSim) maxMeanSim = *itr;
		}

		vector<int> candidates;

		for (vector<double>::const_iterator itr = meanSim.begin(); itr != meanSim.end(); ++itr)
		{
			if (*itr == maxMeanSim)
			{
				candidates.push_back(static_cast<int>(itr - meanSim.begin()));
			}
		}

		if (1 == candidates.size())
		{
			return classLabels[candidates.front()];
		}
		else
		{
			//srand(static_cast<unsigned int>(time(NULL)));			
			//return classLabels[candidates[rand() % candidates.size()]];
			haruki::CmRandomNumberGenerator *pRandomNumberGenerator =
				haruki::CmRandomNumberGenerator::getInstance();
			return classLabels[candidates[pRandomNumberGenerator->getUInt() % candidates.size()]];
		}
	}
}
